==> mutt-0.48.1-h7.4.tgz: mutt-0.48.1-h7.4/sort.c <==
/*
 * Copyright (C) 1996 Michael R. Elkins.
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#include "muttlib.h"
#include "sort.h"

#include <stdlib.h>
#include <string.h>

static int days_per_month[12] = {
  31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
};

#define is_leap_year(x) ((x)%4 == 0 && ((x)%100 != 0))

/* return the number of secs since Jan 1, 1971 12:00am GMT */
static long mygmtime (HEADER *h) {
  long t = 0;
  int i;

  for (i=0; i<h->year; i++) {
    t += 365 * 24 * 3600;
    if (is_leap_year(i+1971)) t += 24 * 3600;
  }
  for (i=0; i<h->month; i++) {
    t += days_per_month[i] * 24 * 3600;
  }
  t += h->day * 24 * 3600;
  t += h->hours * 3600;
  t += h->minutes * 60;
  t += h->seconds;
  if (h->zoccident) { /* west of GMT */
    t += h->zhours * 3600;
    t += h->zminutes * 60;
  }
  else { /* east of GMT */
    t -= h->zhours * 3600;
    t -= h->zminutes * 60;
  }
  return t;
}

#define SORTCODE(x) Sort & SORTREVERSE ? -(x) : x

int compare_size (const void *a, const void *b)
{
  HEADER **ppa = (HEADER **)a;
  HEADER **ppb = (HEADER **)b;
  HEADER *pa = *ppa, *pb = *ppb;
  int rc;

  if (pa->length < pb->length)
    rc = -1;
  else if (pa->length == pb->length)
    rc = 0;
  else
    rc = 1;
  return(SORTCODE(rc));
}

int compare_date_sent (const void *a, const void *b)
{
  HEADER **ppa = (HEADER **)a;
  HEADER **ppb = (HEADER **)b;
  HEADER *pa = *ppa, *pb = *ppb;
  time_t at, bt;
  int rc;
  
  at = mygmtime(pa);
  bt = mygmtime(pb);
  if (at < bt)
    rc = -1;
  else if (at == bt)
    rc = 0;
  else
    rc = 1;
  return (SORTCODE(rc));
}

int compare_subject (const void *a, const void *b)
{
  HEADER **ppa = (HEADER **)a;
  HEADER **ppb = (HEADER **)b;
  HEADER *pa = *ppa, *pb = *ppb;
  int rc;

  if (!pa->env->subject) {
    if (!pb->env->subject)
      rc = compare_date_sent(ppa, ppb);
    else
      rc = -1;
  }
  else if (!pb->env->subject)
    rc = 1;
  else {
    char *a = pa->env->subject;
    char *b = pb->env->subject;

    if (strncasecmp("re:", a, 3) == 0)
      a = skip_whitespace(a+3);
    if (strncasecmp("re:", b, 3) == 0)
      b = skip_whitespace(b+3);
    rc = strcmp(a, b);
  }
  return(SORTCODE(rc));
}

int compare_from (const void *a, const void *b)
{
  HEADER **ppa = (HEADER **)a;
  HEADER **ppb = (HEADER **)b;
  HEADER *pa = *ppa, *pb = *ppb;
  char buf1[STRING] = { 0 };
  char buf2[STRING] = { 0 };

  mutt_write_address(buf1, sizeof(buf1), pa->env->from);
  mutt_write_address(buf2, sizeof(buf2), pb->env->from);
  return(SORTCODE(strcmp(buf1,buf2)));
}

int compare_date_received (const void *a, const void *b)
{
  HEADER **ppa = (HEADER **)a;
  HEADER **ppb = (HEADER **)b;
  HEADER *pa = *ppa, *pb = *ppb;
  int rc;
  
  if (pa->received < pb->received)
    rc = -1;
  else if (pa->received == pb->received)
    rc = 0;
  else
    rc = 1;
  return(SORTCODE(rc));
}

int compare_order (const void *a, const void *b)
{
  HEADER **ha = (HEADER**)a;
  HEADER **hb = (HEADER**)b;

  if ((*ha)->index < (*hb)->index) return(-1);
  return(1);
}

sort_t *mutt_get_sort_func (int method)
{
  switch (method & SORTMASK) {
  case SORTRECEIVED:
    return(compare_date_received);
  case SORTORDER:
    return(compare_order);
  case SORTDATE:
    return(compare_date_sent);
  case SORTSUBJECT:
    return(compare_subject);
  case SORTFROM:
    return(compare_from);
  case SORTSIZE:
    return(compare_size);
  default:
    return 0;
  }
  /* not reached */
}

void mutt_sort_headers (CONTEXT *ctx)
{
  int i;
  sort_t *sortfunc = mutt_get_sort_func(Sort);
  
  if (!ctx || !ctx->msgcount) return; /* nothing to do! */

  mutt_error("Sorting mailbox...");

  if (Sort == SORTTHREADS)
    mutt_sort_threads(ctx);
  else if (!sortfunc) {
    mutt_error("Could not find sorting function! [report this bug]");
    return;
  }
  else 
    qsort((void *)ctx->hdrs, ctx->msgcount, sizeof(HEADER *), sortfunc);

  /* adjust the virtual message numbers */
  ctx->vcount = 0;
  for (i=0; i<ctx->msgcount; i++) {
    if (ctx->hdrs[i]->visible) {
      ctx->hdrs[i]->virtual = ctx->vcount;
      *ctx->v2r[ctx->vcount] = i;
      ctx->vcount++;
    }
    ctx->hdrs[i]->msgno = i+1;
  }

  mutt_error("");
}

==> mutt-0.56-h8.1.tgz: mutt-0.56-h8.1/sort.c <==
/*
 * Copyright (C) 1996 Michael R. Elkins <me@cs.hmc.edu>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#include "muttlib.h"

#include <stdlib.h>
#include <string.h>

#define SORTCODE(x) Sort & SORTREVERSE ? -(x) : x

int compare_size (const void *a, const void *b)
{
  HEADER **ppa = (HEADER **)a;
  HEADER **ppb = (HEADER **)b;
  HEADER *pa = *ppa, *pb = *ppb;
  int rc;

  if (pa->content->length < pb->content->length)
    rc = -1;
  else if (pa->content->length == pb->content->length)
    rc = 0;
  else
    rc = 1;
  return(SORTCODE(rc));
}

int compare_date_sent (const void *a, const void *b)
{
  HEADER **ppa = (HEADER **)a;
  HEADER **ppb = (HEADER **)b;
  HEADER *pa = *ppa, *pb = *ppb;
  time_t at, bt;
  int rc;
  
  at = mutt_gmtime(pa);
  bt = mutt_gmtime(pb);
  if (at < bt)
    rc = -1;
  else if (at == bt)
    rc = 0;
  else
    rc = 1;
  return (SORTCODE(rc));
}

int compare_subject (const void *a, const void *b)
{
  HEADER **ppa = (HEADER **)a;
  HEADER **ppb = (HEADER **)b;
  HEADER *pa = *ppa, *pb = *ppb;
  int rc;

  if (!pa->env->subject) {
    if (!pb->env->subject)
      rc = compare_date_sent(ppa, ppb);
    else
      rc = -1;
  }
  else if (!pb->env->subject)
    rc = 1;
  else {
    char *a = pa->env->subject;
    char *b = pb->env->subject;

    if (strncasecmp("re:", a, 3) == 0)
      a = skip_whitespace(a+3);
    if (strncasecmp("re:", b, 3) == 0)
      b = skip_whitespace(b+3);
    rc = strcmp(a, b);
  }
  return(SORTCODE(rc));
}

int compare_from (const void *a, const void *b)
{
  HEADER **ppa = (HEADER **)a;
  HEADER **ppb = (HEADER **)b;
  HEADER *pa = *ppa, *pb = *ppb;
  char buf1[STRING] = { 0 };
  char buf2[STRING] = { 0 };

  mutt_write_address(buf1, sizeof(buf1), pa->env->from);
  mutt_write_address(buf2, sizeof(buf2), pb->env->from);
  return(SORTCODE(strcmp(buf1,buf2)));
}

int compare_date_received (const void *a, const void *b)
{
  HEADER **ppa = (HEADER **)a;
  HEADER **ppb = (HEADER **)b;
  HEADER *pa = *ppa, *pb = *ppb;
  int rc;
  
  if (pa->received < pb->received)
    rc = -1;
  else if (pa->received == pb->received)
    rc = 0;
  else
    rc = 1;
  return(SORTCODE(rc));
}

int compare_order (const void *a, const void *b)
{
  HEADER **ha = (HEADER**)a;
  HEADER **hb = (HEADER**)b;

  if ((*ha)->index < (*hb)->index) return(-1);
  return(1);
}

sort_t *mutt_get_sort_func (int method)
{
  switch (method & SORTMASK) {
  case SORTRECEIVED:
    return(compare_date_received);
  case SORTORDER:
    return(compare_order);
  case SORTDATE:
    return(compare_date_sent);
  case SORTSUBJECT:
    return(compare_subject);
  case SORTFROM:
    return(compare_from);
  case SORTSIZE:
    return(compare_size);
  default:
    return 0;
  }
  /* not reached */
}

void mutt_sort_headers (CONTEXT *ctx)
{
  int i;
  sort_t *sortfunc = mutt_get_sort_func(Sort);
  
  if (!ctx->msgcount) {
    /*
     * this function gets called by mutt_sync_mailbox(), which may have just
     * deleted all the messages.  the virtual message numbers are not updated
     * in that routine, so we must make sure to zero the vcount member.
     */
    ctx->vcount = 0;
    return; /* nothing to do! */
  }

  mutt_error("Sorting mailbox...");

  if (Sort == SORTTHREADS)
    mutt_sort_threads(ctx);
  else if (!sortfunc) {
    mutt_error("Could not find sorting function! [report this bug]");
    return;
  }
  else 
    qsort((void *)ctx->hdrs, ctx->msgcount, sizeof(HEADER *), sortfunc);

  /* adjust the virtual message numbers */
  ctx->vcount = 0;
  for (i=0; i<ctx->msgcount; i++) {
    if (ctx->hdrs[i]->virtual != -1) {
      ctx->hdrs[i]->virtual = ctx->vcount;
      ctx->v2r[ctx->vcount] = i;
      ctx->vcount++;
    }
    ctx->hdrs[i]->msgno = i+1;
  }

  mutt_error("");
}
